package deploy;

import com.sas.entities.GUID;
import com.sas.metadata.remote.Tree;
import com.sas.metadata.remote.AssociationList;
import com.sas.metadata.remote.SoftwareComponent;
import com.sas.metadata.remote.Document;
import com.sas.metadata.remote.MetadataObjects;
import com.sas.metadata.remote.AccessControlEntry;
import com.sas.metadata.remote.AccessControlTemplate;
import com.sas.metadata.remote.CMetadata;
import com.sas.metadata.remote.ExternalIdentity;


import java.util.List;

public class FolderUtils
{
    private JomaSupport joma;
    private String foundReposId;
    
    public FolderUtils(JomaSupport joma)
    {
        this.joma = joma;
        foundReposId = joma.getFoundationRepositoryId();
    }
    
    
    public String getPath(Document document)
    {
        String path = "";
        if(document == null)
            return path;
        
        AssociationList trees = document.getTrees();
        if(trees != null && trees.size() == 1)        
        {
            path = getPath(trees.get(0));
        }
        return path;
    }
    
    
    public String getPath(Tree tree)
    {
        if(tree == null)
            return "";
        String path = "/" + tree.getName();
        Tree parent = tree.getParentTree();
        if(parent != null)
            path = getPath(parent) + path;
        return path;
    }
    
    
    public Tree makeFolder(Tree srcRoot, Tree targetRoot, String path)
    {
        Tree result = null;
        String [] folders = parsePath(path);
        
        Tree parentSrcFolder = srcRoot;
        Tree parentTargetFolder = targetRoot;
        for(int i = 0; i < folders.length; i++)
        {
            Tree srcFolder = getFolderFromPath(parentSrcFolder, folders[i]);            
            Tree targetFolder = getFolderFromPath(parentTargetFolder, folders[i]); 
            if(targetFolder == null)
            {
                targetFolder = makeFolder(parentTargetFolder, folders[i]);                
                updateTreeGUID(srcFolder, targetFolder);
                updateTreeAccessControls(srcFolder, targetFolder);
            }
            
            if(srcFolder == null || targetFolder == null)
            {
                //THIS REALLY SHOULD NOT HAPPEN, BUT IF IT DOES I THINK WE NEED TO BAIL
                return null;
            }

            parentSrcFolder = srcFolder;
            parentTargetFolder = targetFolder; 
            result = targetFolder
        }
        return result;
    }
    
    
    public Tree makeFolder(Tree root, String path)
    {        
        Tree result = null;
        String parentPath = getParentPath(path);
        if(parentPath.equals(""))
        {
            //CREATE A TOP LEVEL FOLDER IN THE FOUNDATION REPOSITORY
            result = createBIPFolder(null, parsePath(path)[0], root);            
        }
        else
        {            
            Tree parent = getFolderFromPath(root, parentPath);
            if(parent == null)
            {
                parent = makeFolder(root, parentPath);
            }
            
            String [] folders = parsePath(path);
            
            result = createBIPFolder(null, folders[folders.length - 1], parent);
        }
        
        return result;        
    }
   
    
    public Tree makeFolder(String path)
    {        
        return makeFolder(null, path);        
    }
    
    
    public boolean folderExists(String folderPath)
    {
        return getFolderFromPath(folderPath) != null;
    }
    
    
    public boolean documentExists(String repositoryID, String name, String guid, String target)
    {
        String documentSelector = "Document[@Name='" + name + "']";     
        String treeSelector = "[Trees/Tree[@Id='" + target + "']]";
        String guidSelector = "[ExternalIdentities/ExternalIdentity[@Identifier='" + guid + "']]";
    
        List l = joma.getMetadataObjects(repositoryID, "Document", documentSelector + treeSelector + guidSelector);
        
        return l.size() > 0;              
    }
    
    
    public Tree getFolderFromPath(Tree root, String folderPath)
    {
        String[] path = parsePath(folderPath);
        Tree currentFolder = root;
        
        if(path == null || path.length == 0)
            return null;
        
        int start = 0;
        
        if(currentFolder == null)
        {
            //GET THE BIP SERVICE FROM THE FOUNDATION REPOSITORY
            SoftwareComponent sc = joma.getSoftwareComponentByName(null, "BIP Service");
            
            String topLevelName = path[0];
            start = 1;
            
            //GET THE SOFTWARETREE FOR THE FIRST PART OF THE PATH
            AssociationList softwareTrees = sc.getSoftwareTrees();
            for(int i = 0; i < softwareTrees.size(); i ++)
            {
                Tree tree = (Tree) softwareTrees.get(i);
                if(tree.getName().equals(topLevelName))
                {
                    currentFolder = tree;
                    break;
                }
            }
        }
        
        if(currentFolder == null)
            return currentFolder;
        
        //GET THE REST OF THE TREE
        for (int i = start; i < path.length; i++ ) 
        {
            if(currentFolder != null)
                currentFolder = joma.getAssocObjByName(currentFolder, "SubTrees", path[i]);
            else
                return currentFolder;
        }
        
        return currentFolder;
    }
    
    
    public Tree getFolderFromPath(String folderPath)
    {
        return getFolderFromPath(null, folderPath);
    }
    
    
    public Tree createBIPFolder(String repositoryName, String folderName, Tree parentTree)
    {       
        String repositoryID = joma.getRepositoryId(repositoryName);
        Tree tree = (Tree) joma.createObject(repositoryID, MetadataObjects.TREE, folderName);
        
        //SET THE PUBLIC TYPE ATTRIBUTE TO "Folder"
        tree.setPublicType("Folder");
        
        //SET THE USAGE VERSION UsageVersion TO 1000000
        tree.setUsageVersion(1000000);
        
        //SET THE TreeType TO "BIP Folder"
        tree.setTreeType("BIP Folder");
        
        if(parentTree == null)
        {
            //Set the BIP Service SoftwareComponents association
            SoftwareComponent bipService = (SoftwareComponent) joma.getSoftwareComponentByName(null, "BIP Service");
            AssociationList softwareComponents = new AssociationList("SoftwareComponents");
            softwareComponents.add(bipService);
            tree.setSoftwareComponents(softwareComponents);
        }
        else
        {
            tree.setParentTree(parentTree);
            AssociationList subtrees = parentTree.getSubTrees();
            subtrees.add(tree);
            parentTree.setSubTrees(subtrees);
            parentTree.updateMetadataAll();
        }
        
        //UPDATE METADATA
        tree.updateMetadataAll();
        
        CommonUtils.info("CREATED FOLDER: " + folderName);
        
        return tree;
    }
    
    
    public static String getParentPath(String path)
    {
        if (path.endsWith("/")) {
            path = path.substring(0, path.length() - 1); // skip ending slash
        }
        int index = path.lastIndexOf("/");
        return path.substring(0, index == -1 ? 0 : index);
    }
    
    
    public static String[] parsePath(String folderPath)
    {
        if (folderPath.startsWith("/")) {
            folderPath = folderPath.substring(1); // skip leading slash
        }
        return folderPath.split("/");
    }
 
    //////////////////////////////////////////////////////////////////////
    //  Update ExternalIdentity(s)
    //////////////////////////////////////////////////////////////////////
    public void updateTreeGUID(Tree srcTree, Tree trgTree)
    {
        updateGUID(srcTree, trgTree);
    }

    public void updateDocumentGUID(Document srcDoc, Document trgDoc)
    {
        updateGUID(srcDoc, trgDoc);
    }

    public void updateGUID(CMetadata srcObj, CMetadata trgObj)
    {
        String guid = "";
        String name = "guid";
        String context = "GUID";
        
        // Find GUID from Source Object, then delete the source GUID object.
        // It critical that the source GUID object be delete, otherwise, when
        // Doc Mgr attempts to look up objects it will likely find dupes.
        if (srcObj == null)
        {
            // There should be a GUID object here.  Not much we can do, 
            // but to create a new GUID
            guid = GUID.newGUID();
        }
        else
        {
            AssociationList objs = srcObj.getExternalIdentities();
            if (objs == null || objs.isEmpty())
            {
                // There should be a GUID object here.  Not much we can do, 
                // but to create a new GUID
                guid = GUID.newGUID();
            }
            else
            {
                // There should be only one ExternalIdentity object.  If there
                // is more than one, well there isn't much that we can do other
                // than use one and then delete them all
                List objsToDel = new ArrayList();
                for (Iterator it = objs.iterator(); it.hasNext(); )
                {
                    CMetadata obj = (CMetadata) it.next();
                    if(obj.getCMetadataType().equals("ExternalIdentity"))
                    {
                        ExternalIdentity ei = (ExternalIdentity) obj;
                        name = ei.getName();
                        context = ei.getContext();
                        guid = ei.getIdentifier();
                        objsToDel.add(ei);
                    }
                }
                joma.deleteObjects(objsToDel);
            }
        }
        
        // Create new ExternalIdentity object for the Target Object
        CMetadata obj = joma.createObjectNoUpdate(foundReposId, 
                MetadataObjects.EXTERNALIDENTITY, name);
        ExternalIdentity ei = (ExternalIdentity) obj;
        ei.setContext(context);
        ei.setIdentifier(guid);
        ei.setOwningObject(trgObj);
        ei.updateMetadataAll();
        
    }

    //////////////////////////////////////////////////////////////////////
    //  Update AccessControlEntry(s)
    //////////////////////////////////////////////////////////////////////
    public void updateTreeAccessControls(Tree srcTree, Tree trgTree)
    {
        updateAccessControls(srcTree, trgTree);
    }

    public void updateDocumentAccessControls(Document srcDoc, Document trgDoc)
    {
        updateAccessControls(srcDoc, trgDoc);
    }

    public void updateAccessControls(CMetadata srcObj, CMetadata trgObj)
    {

        // Find ACE(s) from Source Object, then delete the source ACE 
        // object(s).
        AssociationList objs = srcObj.getAccessControls();
        if (objs != null && objs.size() > 0)
        {
            List objsToDel = new ArrayList();
            for (Iterator it = objs.iterator(); it.hasNext(); )
            {
                CMetadata obj = (CMetadata) it.next();
                objsToDel.add(obj);
                
                // We could have "AccessControlTemplate"(s) or
                // "AccessControlEntry"(s)
                if(obj.getCMetadataType().equals("AccessControlTemplate"))
                {
                    AccessControlTemplate act = (AccessControlTemplate) obj;
                    createACEFromACT(act, objsToDel, trgObj);
                }
                else if(obj.getCMetadataType().equals("AccessControlEntry"))
                {
                    // Create new ACE on target object
                    AccessControlEntry ace = createACE(obj);
                    AssociationList objects = new AssociationList("Objects");
                    objects.add(trgObj);
                    ace.setObjects(objects);
                    ace.updateMetadataAll();
                }
            }
            joma.deleteObjects(objsToDel);
        }
    }
    
    private void createACEFromACT(AccessControlTemplate act, List objsToDel,
            CMetadata trgObj)
    {
        objsToDel.add(act);
        AssociationList srcACEs = act.getAccessControlItems();
        for (Iterator it = srcACEs.iterator(); it.hasNext(); )
        {
            CMetadata obj = (CMetadata) it.next();
            objsToDel.add(obj);
            
            if(obj.getCMetadataType().equals("AccessControlEntry"))
            {
                // Create new ACE on target object
                AccessControlEntry srcACE = (AccessControlEntry) obj;
                AccessControlEntry ace = createACE(srcACE);
                AssociationList objects = new AssociationList("Objects");
                objects.add(trgObj);
                ace.setAccessControlTemplates(objects);
                ace.updateMetadataAll();
            }
            
        }
    }
    
    private AccessControlEntry createACE(CMetadata srcObj)
    {
        AccessControlEntry srcACE = (AccessControlEntry) srcObj;
        String name = srcACE.getName();
        
        AssociationList identities = srcACE.getIdentities();
        AssociationList newIdents = new AssociationList("Identities");
        for (Iterator it = identities.iterator(); it.hasNext(); )
        {
            CMetadata cmeta = (CMetadata) it.next();
            newIdents.add(cmeta);
        }
        
        AssociationList permissions = srcACE.getPermissions();
        AssociationList newPerms = new AssociationList("Permissions");
        for (Iterator it = permissions.iterator(); it.hasNext(); )
        {
            CMetadata cmeta = (CMetadata) it.next();
            newPerms.add(cmeta);
        }
        
        // Create ACE for target object
        CMetadata cmeta = joma.createObjectNoUpdate(foundReposId, 
                MetadataObjects.ACCESSCONTROLENTRY, name);
        AccessControlEntry ace = (AccessControlEntry) cmeta;
        ace.setDesc(srcACE.getDesc());
        if (newIdents != null && newIdents.size() > 0)
        {
            ace.setIdentities(newIdents);
        }
        if (newPerms != null && newPerms.size() > 0)
        {
            ace.setPermissions(newPerms);
        }
        ace.updateMetadataAll();
        
        return ace;
    }
    
}