package deploy.ContentTypes;

import com.sas.metadata.MetadataObjects;

import com.sas.metadata.remote.AccessControlEntry;
import com.sas.metadata.remote.AssociationList;
import com.sas.metadata.remote.CMetadata;
import com.sas.metadata.remote.ContentType;
import com.sas.metadata.remote.IdentityGroup;
import com.sas.metadata.remote.Permission;
import com.sas.metadata.remote.Property;
import com.sas.metadata.remote.PropertyGroup;
import com.sas.metadata.remote.SoftwareComponent;
import com.sas.metadata.remote.Tree;
import com.sas.metadata.remote.TypeDefinition;

import deploy.AddMetadata;
import deploy.JomaSupport;
import deploy.MetadataExecution;


import java.lang.RuntimeException;
import java.util.ArrayList;
import java.util.List;


/**
 * Conditionally add ContentTypes. 
 
 *   > For SPM we are only going to called on the migration path to fix up
 *   > the migrated content types.  This code was copied from the Solutions
 *   > midtier install.  For a non-migration install the ContentTypes are
 *   > defined through the extract deploy mechanism.
 *
 *   > Add new Tree object named "ContentTypes" in Foundation repository
 *   > Before adding a new ContentType, check that it doesn't already exist
 *   > If not found, add it and associate to Tree and SoftwareComponent
 *   > If found, just change associations to Tree and SoftwareComponent
 *   > Lastly, look in Solutions Repository (if there is one), for
 *     the Tree object named "ContentTypes", if found delete it.
 *
 * This should be run during configuration (before deployment) and
 * requires (run after) the following metadata already exists:
 *   > Solutions/FM Identities
 *       - SASAdministrators
 *       - PUBLIC
 *       - Administrators
 *       - Analyst
 *       - InformationConsumer
 *   > Solutions SoftwareComponent
 *       - Solutions Svc <release number>
 *
 *
 * @author <a href="mailto:Jon.Segerstrom@sas.com">Jon Segerstrom</a>
 * @author <a href="mailto:Jeremy.Smith@sas.com">Jeremy Smtih</a>

 */

 int rc = 0;
 try
 {
     ContentTypes ct = new ContentTypes(attributes);
     // println("** ** Starting migrateSpmContentType groove ** **");
     ct.install();
 }
 catch (Exception e)
 {
     rc = 1;
     project.setNewProperty("outputproperty", e.toString());
     e.printStackTrace();
 }
 project.setNewProperty("resultsproperty", "" + rc);
 
class ContentTypes
{
    boolean DEBUG = true;
    String user;
    String password;
    String host;
    String port;
    String foundReposNm;
    String solSrvcNm;
    // String contentTypePath;
    
    String solReposNm = "Solutions";
    
    String treeNm = "ContentTypes";
    String grpSASAdminNm = "SASAdministrators";
    String grpPublicNm = "PUBLIC";

    IdentityGroup grpSasAdmin;
    IdentityGroup grpPublic;
    
    Permission grantReadMetadata;
    Permission grantWriteMetadata;
    
    SoftwareComponent solSC;
    
    JomaSupport joma;
    MetadataExecution executor;
  
    String[] ctNames = ["KPIProject", 
                        "SPMProject"];
    
    public ContentTypes(Map attributes)
    {
        this.user = attributes.get("admin.user");
        this.password = attributes.get("admin.pass");
        this.host = attributes.get("omr.host");
        this.port = attributes.get("omr.port");
        this.foundReposNm = attributes.get("foundation.repos.nm");
        this.solSrvcNm = attributes.get("sss.swc.name");
        // this.contentTypePath = attributes.get("config.path") + "ContentTypes";

    }
    
    public void install()
    {
        try
        {
            debug("--- INPUT ATTRIBUTES ---");
            debug("User: " + user);
            debug("Password: " + password);
            debug("OMR Host: " + host);
            debug("OMR Port: " + port);
            debug("Foundation Repos Nm: " + foundReposNm);
            debug("------------------------");
            debug("");

            joma = new JomaSupport();
            joma.connect(host, port, user, password);
            String foundReposId = joma.getRepositoryId(foundReposNm);
            debug("Foundation Repos Id:  " + foundReposId);
            
            // Get Identities
            grpSasAdmin = getIdentityGroup(foundReposId, grpSASAdminNm);
            grpPublic = getIdentityGroup(foundReposId, grpPublicNm);
            
            // Get Permissions
            grantReadMetadata = getPermission(foundReposId, 
                    "ReadMetadata", "Grant");
            grantWriteMetadata = getPermission(foundReposId, 
                    "WriteMetadata", "Grant");
            
            // Get SoftwareComponent
            solSC = getSoftwareComponent(foundReposId, solSrvcNm);
            
            // Get Tree
            Tree foundTree = createContentTypeTree(foundReposId, treeNm);
            
            executor = new MetadataExecution(host, port, user, password);

            // On a 9.1.3 migrated install, we should find a Tree object
            // of name "ContentTypes" in the Solutions repository.  This
            // Tree object will have associated ContentType objects.  We'll
            // need to:
            //    > Delete the following ContentTypes:
            //         - GeographicAnalysis    (HCM)
            //         - OrgChart              (HCM)
            //         - ProfileDocument       (HCM)
            //         - SimOrgChart           (HCM)
            //         - TrashCan             
            //         - WebReportStudio 
            //    > Associate remaining ContentType objects to Tree 
            //      object in Foundation
            //    > Delete the Tree object in Solutions
            //
            // If Solutions Tree object is not found, then ContentTypes
            // are presumed not to exist (i.e. new install) and need to be
            // created
            String solReposId = joma.getRepositoryId(solReposNm);
            debug(solReposNm + " Repository Id:  " + solReposId);
            if (solReposId == null)
            {
                debug("***** New install, start of adding ContentTypes *****");
                updateContentTypes(foundReposId, foundTree);
            }
            else
            {
                debug("***** Migration, start of spmContentType update *****");
                // deleteContentTypes(foundReposId);  // Done in Solutions Midtier migration
                updateContentTypes(foundReposId, foundTree);
                // deleteTree(solReposId, treeNm);    // Done in Solutions Midtier migration
            } 

        }
        catch (Exception e)
        {
          throw e;
        }
        finally
        {
            if (joma != null)
            {
                joma.disconnect();
            }
            if (executor != null)
            {
                executor.dispose();
            }
        }
    }
    
    private void deleteTree(String reposId, String nm)
    {
        List delObjs = new ArrayList();
        
        String xmlSelect = getXMLSelect(nm);
        String template = "<Templates name=''>" + 
                             "<Tree>" +
                               "<AccessControls/>" +
                             "</Tree>" +
                          "</Templates>";

        List trees = joma.getMetadataObjects(reposId, MetadataObjects.TREE, 
                xmlSelect, template);
        
        for (Iterator it = trees.iterator(); it.hasNext(); )
        {
            Tree tree = (Tree) it.next();
            delObjs.add(tree);
            debug("Deleting Tree obj Name: " + nm + ", Id: " + tree.getFQID() + 
                    ", Repos Id: " + reposId);
            
            AssociationList aces = tree.getAccessControls();
            delObjs.addAll(aces);
            
        }
        
        if (!delObjs.isEmpty())
        {
            joma.deleteObjects(delObjs);
        }
    }
    
    private void updateContentTypes(String reposId, Tree tree)
    {
        for (int i = 0; i < this.ctNames.length; i++)
        {
            updateContentType(reposId, this.ctNames[i], tree);
        }

    }
    
    private void updateContentType(String reposId, String nm, 
            Tree tree)
    {
        List objs = getContentTypes(reposId, nm);
        if (objs.size() == 0)
        {
            // createContentType(reposId, nm, tree);  // For Spm these are extract
            throw new RuntimeException("Error: No" + 
	      " ContentType object(s) of name: " + nm + ", was " + 
	                        "found in repos id: " + reposId);

        }
        else if (objs.size() > 1)
        {
            throw new RuntimeException("Error: " + objs.size() + 
                    " ContentType object(s) of name: " + nm + ", was " + 
                    "found in repos id: " + reposId);
        }
        else
        {
            addTreeAssociation((ContentType) objs.get(0), tree);
            
            TypeDefinition typedef =  getTypeDefinition(reposId, nm);
            addTypeDefinitionAssociation((ContentType) objs.get(0), typedef );
        }

    }
    
    private void addTreeAssociation(ContentType obj, Tree tree)
    {
        AssociationList trees = obj.getTrees();
        trees.clear();
        trees.add(tree);
        obj.setTrees(trees);
        obj.updateMetadataAll();
        debug("Set Tree Association on Content Type obj Name: " + obj.getName() + 
                ", Id: " + obj.getFQID() + " to -> Tree obj Name: " + 
                tree.getName() + ", Id: " + tree.getFQID());
    }
 
    private void addTypeDefinitionAssociation(ContentType obj, TypeDefinition typeDef)
    {
        AssociationList typeDefinitions = obj.getTypeDefinitions();
        typeDefinitions.clear();
        typeDefinitions.add(typeDef);
        obj.setTypeDefinitions(typeDefinitions);
        obj.updateMetadataAll();
        debug("Set TypeDefinition on Content Type obj Name: " + obj.getName() + 
                ", Id: " + obj.getFQID() + " to -> TypeDefinition obj Name: " + 
                typeDef.getName() + ", Id: " + typeDef.getFQID());
    }    
    
 //   private void createContentType(String reposId, String nm, Tree tree)
 //   {
 //       debug("Creating ContentType obj Name: " + nm + ", in Repos Id: " + 
 //               reposId);
 //       
 //     AddMetadata adder = new AddMetadata();
 //     adder.RUN_UPDATES = true;
 //       
 //     String ctXML = this.contentTypePath + "/" + nm + ".xml";
 //     String rawXML = adder.addRawMetadata(reposId, ctXML, 
 //              new Hashtable(), executor);
 //     executor.execute(rawXML, "ContentType");

 //       // Fetch the new object as add Tree Association
 //     List objs = getContentTypes(reposId, nm);
 //       addTreeAssociation((ContentType)objs.get(0), tree);
 //       
 //  }
    
    //  Delete the following ContentTypes:
    //         - GeographicAnalysis    (HCM)
    //         - OrgChart              (HCM)
    //         - ProfileDocument       (HCM)
    //         - SimOrgChart           (HCM)
    //         - TrashCan             
    //         - WebReportStudio 
    //
    //  At this point in an Install, HCM Documents should
    //  have already migrated (occurs when HCM Deploys).
    //
    //  Further, WebReportSutdio content will have been
    //  migrated as well
    //
    //TODO:  Delete associated TrashCan objects
    private void deleteContentTypes(String reposId)
    {
        // Delete un-supported ContentType Objects
        // deleteContentType(reposId, "GeographicAnalysis");
        // deleteContentType(reposId, "OrgChart");
        // deleteContentType(reposId, "ProfileDocument");
        // deleteContentType(reposId, "SimOrgChart");
        // deleteContentType(reposId, "WebReportStudio");
        // deleteContentType(reposId, "TrashCan");
        
    }
    
    private void deleteContentType(String reposId, String nm)
    {
        List objs = getContentTypes(reposId, nm);
        if (objs.size() > 0)
        {
            for (Iterator it = objs.iterator(); it.hasNext(); )
            {
                ContentType obj = (ContentType) it.next();
                debug("Deleting ContentType Object Name: " + nm + ", Id: " + 
                        obj.getFQID() + ", from Repository Id: " + reposId);
            }
            joma.deleteObjects(objs);
        }
    }

    private List getContentTypes(String reposId, String nm)
    {
        String xmlSelect = getXMLSelect(nm);

        List objs = joma.getMetadataObjects(reposId, 
                MetadataObjects.CONTENTTYPE, xmlSelect);

        for (Iterator it = objs.iterator(); it.hasNext(); )
        {
            ContentType obj = (ContentType) it.next();
            debug("Found ContentType object of name: " + nm +
                    ", in repository id: " + reposId + 
                    ", obj id: " + obj.getFQID());
        }
        return objs;
    }
    
    private IdentityGroup getIdentityGroup(String reposId, String nm)
    {
        String xmlSelect = getXMLSelect(nm);

        List groups = joma.getMetadataObjects(reposId, 
                MetadataObjects.IDENTITYGROUP, xmlSelect);
        
        if (groups.size() == 1)
        {
            IdentityGroup group = (IdentityGroup) groups.get(0);
            debug("Found IdentityGroup object of name: " + nm +
                    ", in repository id: " + reposId + 
                    ", obj id: " + group.getFQID());
            return group;
        }
        else
        {
            throw new RuntimeException("Error: " + groups.size() + 
                    " IdentityGroup object(s) of name: " + nm + ", was " + 
                    "found in repos id: " + reposId);
        }
    }
    
    private Permission getPermission(String reposId, String nm, String type)
    {
        String xmlSelect = getXMLSelect(nm) + " AND @Type='" + type + "'";

        List permissions = joma.getMetadataObjects(reposId, 
                MetadataObjects.PERMISSION, xmlSelect);
        
        if (permissions.size() == 1)
        {
            Permission permission = (Permission) permissions.get(0);
            debug("Found Permission object of name: " + nm +
                    ", in repository id: " + reposId + 
                    ", obj id: " + permission.getFQID());
            return permission;
        }
        else
        {
            throw new RuntimeException("Error: " + permissions.size() + 
                    " Permission object(s) of name: " + nm + ", was " + 
                    "found in repos id: " + reposId);
        }
    }
    
    private SoftwareComponent getSoftwareComponent(String reposId, String nm)
    {
        String xmlSelect = getXMLSelect(nm);

        List scs = joma.getMetadataObjects(reposId, 
                MetadataObjects.SOFTWARECOMPONENT, xmlSelect);
        
        if (scs.size() == 1)
        {
            SoftwareComponent sc = (SoftwareComponent) scs.get(0);
            debug("Found SoftwareComponent object of name: " + nm +
                    ", in repository id: " + reposId + 
                    ", obj id: " + sc.getFQID());
            return sc;
        }
        else
        {
            throw new RuntimeException("Error: " + scs.size() + 
                    " SoftwareComponent object(s) of name: " + nm + ", was " + 
                    "found in repos id: " + reposId);
        }
    }
    
    private Tree createContentTypeTree(String reposId, String nm)
    {
        Tree tree = getContentTypeTree(reposId, nm);
        if (tree != null)
        {
            return tree;
        }
        
        // Create Public and SASAdmin ACE for Tree object
        AssociationList identities = new AssociationList("Identities");
        identities.add(this.grpPublic);
        AssociationList permissions = new AssociationList("Permissions");
        permissions.add(this.grantReadMetadata);
        permissions.add(this.grantWriteMetadata);
        AccessControlEntry publicACE = createACE(reposId, grpPublicNm, 
                identities, permissions);
        
        identities = new AssociationList("Identities");
        identities.add(this.grpSasAdmin);
        // For some reason, we need to re-declare this list (presumed
        // to be cleared in above createACE
        permissions = new AssociationList("Permissions");
        permissions.add(this.grantReadMetadata);
        permissions.add(this.grantWriteMetadata);
        AccessControlEntry sasAdminACE = createACE(reposId, grpSASAdminNm, 
                identities, permissions);
        AssociationList aces = new AssociationList("AccessControls");
        aces.add(publicACE);
        aces.add(sasAdminACE);
        
        // Add Solutions SoftwareComponent Association
        AssociationList scs = new AssociationList("SoftwareComponents");
        scs.add(solSC);
        
        // Add Tree object
        CMetadata obj = joma.createObject(reposId, MetadataObjects.TREE, nm);
        tree = (Tree) obj;
        tree.setPublicType("Folder");
        tree.setUsageVersion("1000000");
        tree.setAccessControls(aces);
        tree.setSoftwareComponents(scs);
        tree.updateMetadataAll();
        
        return tree;
    }
    
    private Tree getContentTypeTree(String reposId, String nm)
    {
        String xmlSelect = getXMLSelect(nm);
        String template = "<Templates name=''>" + 
                             "<Tree>" +
                               "<Members/>" +
                             "</Tree>" +
                          "</Templates>";

        List trees = joma.getMetadataObjects(reposId, MetadataObjects.TREE, 
                xmlSelect, template);
        
        if (trees.size() == 0)
        {
            debug("No Tree object found, of name: " + nm + 
                    ", in repository id: " + reposId);
            return null;
        }
        else if (trees.size() == 1)
        {
            Tree tree = (Tree) trees.get(0);
            debug("Found Tree object, in repository id: " + reposId + 
                    ", Tree obj id: " + tree.getName());
            return tree;
        }
        else
        {
            throw new RuntimeException("Error: " + trees.size() + 
                    " Tree object(s) of name: " + nm + ", was " + 
                    "found in repos id: " + reposId);
        }
                               
    }

    private TypeDefinition getTypeDefinition(String reposId, String nm)
    {
        String xmlSelect = "@TypeName='" + nm + "'"; // getXMLSelect(nm);
 
        List typedefs = joma.getMetadataObjects(reposId, MetadataObjects.TYPEDEFINITION, 
                xmlSelect);
        
        if (typedefs.size() == 0)
        {
            debug("No TypeDefinition object found, of TypeName: " + nm + 
                    ", in repository id: " + reposId);
            return null;
        }
        else if (typedefs.size() == 1)
        {
            TypeDefinition typedef = (TypeDefinition) typedefs.get(0);
            debug("Found TypeDefinition object, in repository id: " + reposId + 
                    ", TypeDefinition obj id: " + typedef.getTypeName());
            return typedef;
        }
        else
        {
            throw new RuntimeException("Error: " + typedefs.size() + 
                    " TypeDefinition object(s) of TypeName: " + nm + ", was " + 
                    "found in repos id: " + reposId);
        }
                               
    }    
    
    private AccessControlEntry createACE(String reposId, String nm,
            AssociationList identities, AssociationList permissions)
    {
        CMetadata obj = joma.createObject(reposId, 
                MetadataObjects.ACCESSCONTROLENTRY, nm);
        AccessControlEntry ace = (AccessControlEntry) obj;
        
        ace.setIdentities(identities);
        ace.setPermissions(permissions);
        
        ace.updateMetadataAll();
        return ace;
    }
    
    
    private String getXMLSelect(String nm)
    {
        return "@Name='" + nm + "'";
    }
    
    private void debug(String s)
    {
        if (DEBUG)
        {
            println(s);
        }
    }
    
}
 
